/**
 * aboutKitchenSinkModules.js
 * JavaScript framework for JavaScript-based XPCOM component registration
 * Mook 2004 / 2005
 * License: MPL
 */

// all XPCOM components this can make (all fields are required)
var KnownComponents = [
	{
		Constructor: null,	// the name of the constructor (as a string)
		ContractID: null,	// the contract-id of the component
		CID: null,			// the component id of the component
							// made via Components.ID(GUID-as-a-string)
		Description: null,	// a description of the component (never used?)
		Categories: [		// categories to register to
			{ Category: null,
			  Entry: null }
		]
	},
	{
		Constructor: 'cmdLineTest',
		ContractID: "@minimizetotray.mozdev.org/minimizetotray/cmdlinehandler;1",
		CID: Components.ID("{089ef282-0c7d-437e-a5d1-09d90a872929}"),
		Description: "command line hander test",
		Categories: [
			{ Category: "command-line-handler",
			  Entry: "w-cmdlinetest"
			}
		]
	}
];

// the list of JavaScript files placed in the same directory to read from
// this is used so we can implement the XPCOM components in external files
var ExternalScripts = [
	"minimizeToTrayCmdLine.js"
];

// the name of the module (used only as extra info on component registration)
var ModuleName = "cmdLineTestModule";

////////// IMPLEMENTATION DETAILS BELOW //////////

/**
 * This loads external scripts at run time, in the same directory as this file
 */
(function(){
	var self;
	var fileProtocolHandler =
		Components.classes['@mozilla.org/network/protocol;1?name=file']
			.createInstance(Components.interfaces['nsIFileProtocolHandler']);
	if ("undefined" != typeof(__LOCATION__)) {
		// preferred way
		self = __LOCATION__;
	} else {
		// hacky, but proven :)
		var selfname = Components.Exception().filename;	// get the name of this file
		self = fileProtocolHandler.getFileFromURLSpec(selfname);
	}
	var parent = self.parent; // the directory this file is in
	var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
		.createInstance(Components.interfaces['mozIJSSubScriptLoader']);
	
	for (var index in ExternalScripts) {
		try {
			var child = parent.clone();
			child.append(ExternalScripts[index]);
			// child is a nsILocalFile of the file we want to load
			var childname = fileProtocolHandler.getURLSpecFromFile(child);
			loader.loadSubScript(childname);
		} catch (ex) {
			dump('JavaScript-XPCOM module error:\n');
			dump('While loading external script "' 
				+ ExternalScripts[index] + '":\n');
			dump(ex + '\n');
			try {
				dump('File: ' + ex.filename + '\n');
				dump('Line: ' + ex.lineNumber + '\n');
				if (ex.location) {
					dump('Stack File: ' + ex.location.filename + '\n');
					dump('Stack Line: ' + ex.location.lineNumber + '\n');
					dump('Stack Source: ' + ex.location.sourceLine + '\n');
				}
			} catch (e) {
				// we had problems dumping the error - too bad
				dump('(Error retrieving error location)\n');
			}
			throw(ex);
		}
	}
})();

/**
 * This is the module handler for all JavaScript based XPCOM modules.
 */
var mihoJSModules = {
	_firstTime: true,
	
	registerSelf: function (aComponentManager, aFileSpec, aLocation, aType) {
		var index, info;
		if (this._firstTime) {
			this._firstTime = false;
			throw Components.results['NS_ERROR_FACTORY_REGISTER_AGAIN'];
		}
		aComponentManager = aComponentManager
			.QueryInterface(Components.interfaces['nsIComponentRegistrar']);
			
		var CategoryManager = 
			Components.classes["@mozilla.org/categorymanager;1"]
				.getService(Components.interfaces['nsICategoryManager']);
		
		// No idea how to do this only for debug builds... :p
		dump("*** Registering " + ModuleName
			+ " components (all right -- a JavaScript module!)\n");
		
		for (index in KnownComponents) {
			var info = KnownComponents[index];
			if ("undefined" == typeof(info.CID) || !(info.CID))
				continue;
			aComponentManager.registerFactoryLocation(
				info.CID,
				info.Description,
				info.ContractID,
				aFileSpec,
				aLocation,
				aType
				);
			if ("undefined" == typeof(info['Categories']))
				continue;
			var catindex, cat;
			for (catindex in info['Categories']) {
				cat = info['Categories'][catindex];
				if ("string" != typeof(cat['Category']) || "string" != typeof(cat['Entry']))
					continue;
				CategoryManager.addCategoryEntry(
					cat['Category'],
					cat['Entry'],
					info['ContractID'],
					true,
					true
				);
			}
		}
	},

	getClassObject: function (aComponentManager, aCID, aIID) {
		var index, info;
		for (index in KnownComponents) {
			info = KnownComponents[index];
			if (("undefined" != typeof(info.CID)) && (info.CID) 
			  && (aCID.equals(info.CID)))
				return {
					createInstance: function( aOuter, aIID ) {
						if (aOuter != null)
							throw Components.results['NS_ERROR_NO_AGGREGATION'];
						return (new (eval(info.Constructor)))
							.QueryInterface(aIID);
					},
					lockFactory: function ( aLock ) {
						throw Components.results['NS_ERROR_NOT_IMPLEMENTED'];
					},
					QueryInterface: function( aIID ) {
						if ( aIID.equals(Components.interfaces['nsISupports'])
						  || aIID.equals(Components.interfaces['nsIFactory']))
							return this;
						throw Components.results['NS_ERROR_NO_INTERFACE'];
					}
				};
		}

		if (!aIID.equals(Components.interfaces['nsIFactory']))
			throw Components.results['NS_ERROR_NOT_IMPLEMENTED'];
	
		throw Components.results['NS_ERROR_NO_INTERFACE'];
	},

	canUnload: function (aComponentManager) {
		return true;
	}
};

function NSGetModule( aComponentManager, aFileSpec ) {
	return mihoJSModules;
}